from dabeplech.base import BaseAPI, GETMixin


class PDBeAPI(BaseAPI, GETMixin):
    BASE_URL = "https://www.ebi.ac.uk/pdbe/api/"


class PDBeUniprotMappingAPI(PDBeAPI):
    ROUTE = "mappings/uniprot/"


class PDBePFAMMappingAPI(PDBeAPI):
    ROUTE = "mappings/pfam/"



from .base import (
    BaseAPI, LISTMixin, GETMixin, POSTMixin, PUTMixin
)


class BaseMetageneDBAPI(BaseAPI):
    BASE_URL = 'https://metagenedb.pasteur.cloud/'

    def __init__(self, base_url=BASE_URL, jwt_token=None):
        self.base_url = base_url
        super().__init__()
        if jwt_token is not None:
            self.session.headers.update({
                'Authorization': f"JWT {jwt_token}"
            })


class MetageneDBTokenAPI(BaseMetageneDBAPI, POSTMixin):
    ROUTE = 'api/auth/obtain_token/'


class MetageneDBAPI(BaseMetageneDBAPI, LISTMixin, GETMixin, POSTMixin, PUTMixin):
    pass


class MetageneDBCatalogGeneAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/genes/'


class MetageneDBCatalogTaxonomyAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/taxonomy/'


class MetageneDBCatalogFunctionAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/functions/'


class MetageneDBCatalogKeggOrthologyAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/kegg-orthologies/'


class MetageneDBCatalogEggNOGAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/eggnogs/'


class MetageneDBCatalogStatisticsAPI(MetageneDBAPI):
    ROUTE = 'api/catalog/v1/statistics/'



from typing import List

from pydantic import BaseModel

from dabeplech.models.kegg.references import KeggReferenceModel


class BaseKeggModel(BaseModel):
    entry_id: str
    names: List[str]


class BaseKeggListModel(BaseModel):
    entries: List[BaseKeggModel]


class BaseKeggWithRefModel(BaseKeggModel):
    references: List[KeggReferenceModel] = None



import os
from unittest import TestCase

from dabeplech.parsers.kegg.link import KeggLinkParser


class TestKeggLinkParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/link_pathway_K00135.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggLinkParser(test_txt)
        test_parser.parse()
        tested_entry = test_parser.validated_entry
        # Test some of the attribute and content of the file
        self.assertEqual(len(tested_entry.links), 1)
        for entry_id, entries_list in tested_entry.links.items():
            self.assertEqual(entry_id, 'K00135')
            self.assertEqual(len(entries_list), 14)
            self.assertIn('ko01100', entries_list)



import colored


def _format_print(key_word, msg, color=None):
    """
    Format message such as:
      --> {key_word} ... {msg}
    """
    if color:
        return f" --> {key_word} ... {colored.stylize(msg, colored.fg(color))}"
    return f" --> {key_word} ... {msg}"



import logging

from dabeplech.models.kegg import KeggOrthologyModel
from dabeplech.parsers.kegg.base import BaseKeggParser

logging.basicConfig()
logger = logging.getLogger()


class KeggOrthologyParser(BaseKeggParser):
    """
    Parser for KEGG KO `plain text` result from KEGG API.
    """
    model = KeggOrthologyModel

    def _handle_definition(self, line: str, **kwargs):
        def_and_ec_numbers = line.split(maxsplit=1)[-1]
        if 'EC:' in def_and_ec_numbers:
            elements = def_and_ec_numbers.split('[')
            self.entry.definition = elements[0].strip()
            self.entry.ec_numbers = elements[1].replace(']', '').replace('EC:', '').strip().split()
        else:
            self.entry.definition = def_and_ec_numbers



from urllib.parse import urljoin

from requests.exceptions import HTTPError

from .base import BaseAPI, GETMixin


class TogoWSAPI(BaseAPI, GETMixin):
    BASE_URL = 'http://togows.org'


class TogoWSEntryAPI(TogoWSAPI):
    TYPE = 'entry'
    DATABASES = [
        'ncbi-nuccore',
        'ncbi-nucest',
        'ncbi-nucgss',
        'ncbi-nucleotide',
        'ncbi-protein',
        'ncbi-gene',
        'ncbi-homologene',
        'ncbi-snp',
        'ncbi-mesh',
        'ncbi-pubmed',
        'ebi-ena',
        'ebi-uniprot',
        'ebi-uniparc',
        'ebi-uniref100',
        'ebi-uniref90',
        'ebi-uniref50',
        'ddbj-ddbj',
        'ddbj-dad',
        'pdbj-pdb',
        'kegg-compound',
        'kegg-drug',
        'kegg-enzyme',
        'kegg-genes',
        'kegg-glycan',
        'kegg-orthology',
        'kegg-reaction',
        'kegg-module',
        'kegg-pathway'
    ]
    FORMATS = [
        "json", "fasta", "gff", "ttl", "xml"
    ]

    def __init__(self, database: str, entry_format: str = 'json'):
        """
        :param database: selected target database (list in self.DATABASES)
        :param entry_format: format for the response
        """
        super().__init__()
        if database not in self.DATABASES:
            raise ValueError(f"{database} is not a valid database")
        if entry_format not in self.FORMATS:
            raise ValueError(f"{entry_format} is not a valid entry format")
        self.database = database
        self.format = entry_format
        self.route = f"{self.TYPE}/{self.database}/"
        self.url = urljoin(self.BASE_URL, self.route)

    def get(self, entry_id: str):
        """
        :param entry_id: ID to obtain from selected database
        """
        content = super().get(f"{entry_id}.{self.format}")
        if not content:
            raise HTTPError(f"ID: <{entry_id}> not found in {self.url}")
        return content[0]

    def get_field(self, entry_id: str, field: str):
        """
        :param entry_id: ID to obtain from selected database
        :param field: specific field to select in the response
        """
        return super().get(f"{entry_id}/{field}.{self.format}")



from typing import Union

import logging
from urllib.parse import urljoin

from dabeplech.base import BaseAPI
from dabeplech.parsers.kegg import (
    KeggOrthologyParser, KeggOrthologyListParser,
    KeggPathwayParser, KeggPathwayListParser, KeggLinkParser,
    KeggModuleParser, KeggModuleListParser
)

logging.basicConfig()
logger = logging.getLogger()


class KEGGAPI(BaseAPI):
    ALLOWED_DATABASES: list = [
        'pathway', 'brite', 'module', 'ko', 'genome', 'organism', 'vg', 'ag', 'compound',
        'glycan', 'reaction', 'rclass', 'enzyme', 'network', 'variant', 'disease',
        'drug', 'dgroup', 'environ'
    ]
    BASE_URL: str = 'http://rest.kegg.jp'
    _LIST_PARSER: dict = {
        'ko': KeggOrthologyListParser,
        'pathway': KeggPathwayListParser,
        'module': KeggModuleListParser,
    }
    _GET_PARSER: dict = {
        'ko': KeggOrthologyParser,
        'pathway': KeggPathwayParser,
        'module': KeggModuleParser,
    }

    def _check_db(self, database: str):
        if database not in self.ALLOWED_DATABASES:
            raise Exception(f"<{database}> not a valid database for KEGG. Must choose among {self.ALLOWED_DATABASES}")

    def list(self, database: str, get_model: bool = True) -> Union[dict, str]:
        """
        :param database: selected database you want to retrieve all content from (list in self.ALLOWED_DATABASES)
        :param get_model: return pydantic model (return dict if False)
        :return: response from KEGG API for the database (can be Pydantic model format too)
        """
        self._check_db(database)
        full_url = urljoin(self.url, f"list/{database}")
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        if self._LIST_PARSER.get(database, None) is not None:
            parser = self._LIST_PARSER.get(database)(response.text)
            parser.parse()
            if get_model:
                return parser.validated_model
            return parser.validated_model.dict()
        else:
            logger.warning("Parser not defined yet for %s, returning plain text", database)
        return response.text

    def find(self, database: str, query: str, get_model: bool = True) -> Union[dict, str]:
        """
        :param database: selected database you want to query (list in self.ALLOWED_DATABASES)
        :param query: query you want to find in the chosen db
        :param get_model: return pydantic model (return dict if False)
        :return: response from KEGG API for the database (can be Pydantic model format too)
        """
        self._check_db(database)
        full_url = urljoin(self.url, f"find/{database}/{query}")
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        if self._LIST_PARSER.get(database, None) is not None:
            parser = self._LIST_PARSER.get(database)(response.text)
            parser.parse()
            if get_model:
                return parser.validated_model
            return parser.validated_model.dict()
        else:
            logger.warning("Parser not defined yet for %s, returning plain text", database)
        return response.text

    def _perform_link_request(self, full_url: str):
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        parser = KeggLinkParser(response.text)
        parser.parse()
        return parser.validated_entry

    def link_db(self, target_db: str, source_db: str, get_model: bool = True) -> Union[dict, str]:
        """
        :param target_db: selected target database (list in self.ALLOWED_DATABASES)
        :param source_db: selected source database (list in self.ALLOWED_DATABASES)
        :param get_model: return pydantic model (return dict if False)
        :return: response from KEGG API for the database (can be Pydantic model format too)
        """
        self._check_db(target_db)
        self._check_db(source_db)
        full_url = urljoin(self.url, f"link/{target_db}/{source_db}")
        validated_entry = self._perform_link_request(full_url)
        if get_model:
            return validated_entry
        return validated_entry.dict()

    def link_entries(self, target_db: str, db_entries: str, get_model: bool = True) -> Union[dict, str]:
        """
        :param target_db: selected target database (list in self.ALLOWED_DATABASES)
        :param db_entries: selected entries (several entry can be specified using + operator)
        :param get_model: return pydantic model (return dict if False)
        :return: response from KEGG API for the database (can be Pydantic model format too)
        """
        self._check_db(target_db)
        full_url = urljoin(self.url, f"link/{target_db}/{db_entries}")
        validated_entry = self._perform_link_request(full_url)
        if get_model:
            return validated_entry
        return validated_entry.dict()

    def get(self, entry_id: str, get_model: bool = True) -> Union[dict, str]:
        """
        :param entry_id: KEGG ID to retrieve.
        :param get_model: return pydantic model (return dict if False)
        :return: response from KEGG API for the given entry_id (can be Pydantic model format too)
        """
        full_url = urljoin(self.url, f"get/{entry_id}")
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        database = response.text.splitlines()[0].split()[-1].lower()
        if self._GET_PARSER.get(database, None) is not None:
            parser = self._GET_PARSER.get(database)(response.text)
            parser.parse()
            if get_model:
                return parser.validated_entry
            return parser.validated_entry.dict()
        else:
            logger.warning("Parser not defined yet for %s, returning plain text", database)
        return response.text



from typing import List

from pydantic import Field

from dabeplech.models.kegg.dblinks import DBLinksModel
from .base import BaseKeggListModel, BaseKeggWithRefModel


class KeggPathwayModel(BaseKeggWithRefModel):
    entry_id: str = Field(regex=r'^(map|ko)\d{5}$')
    description: str = None
    classes: List[str] = None
    modules: dict = None
    pathway_maps: dict = None
    orthologies: dict = None
    compounds: dict = None
    related_pathways: dict = None
    ko_pathway: str = Field(None, regex=r'^ko\d{5}$')
    dblinks: DBLinksModel = None
    diseases: dict = None


class KeggPathwayListModel(BaseKeggListModel):
    pass



from dabeplech import HALAPI

from utils import _format_print


def _expected_working_hal():
    tests = [
        ('10.12688/f1000research.12974.1', HALAPI)
    ]
    for i in tests:
        entry_id = i[0]
        api_connector = i[1]
        api = api_connector()
        print(f"Test getting {entry_id} from HAL database from base URL: {api.BASE_URL}:")
        try:
            content = api.find(doi=entry_id)  # noqa
            print(_format_print("Status", "OK", "green"))
            if content:
                print(_format_print(f"Content (format:{type(content)})", "NOT EMPTY", "green"))
            else:
                print(_format_print("Content", "EMPTY", "red"))
        except Exception:
            print(_format_print("Status", "ERROR", "red"))
        finally:
            print(f" --> Requested URL: {api.last_url_requested}")


def _expected_not_working_hal():
    tests = [
        ('10.12688f1000research.12974.1', HALAPI)
    ]
    for i in tests:
        entry_id = i[0]
        api_connector = i[1]
        api = api_connector()
        print(f"Test getting {entry_id} from HAL database from base URL: {api.BASE_URL}:")
        try:
            api = api_connector()
            content = api.get(doi=entry_id)  # noqa
            print(_format_print("Status", "OK", "red"))
        except Exception:
            print(_format_print("Status", "ERROR", "green"))
        finally:
            print(f" --> Requested URL: {api.last_url_requested}")


def test_halapi():
    print("---------- Testing DOI API endpoints... ----------")
    _expected_working_hal()
    _expected_not_working_hal()



from urllib.parse import urljoin

from dabeplech.base import BaseAPI, GETMixin


class HALAPI(BaseAPI, GETMixin):
    BASE_URL = "https://api.archives-ouvertes.fr/search/"

    def find(self, doi: str):
        """
        :param doi: the DOI of the searched document
        :return: response from HAL API
        """
        full_url = urljoin(self.url, f'?fq=doiId_s:"{doi}"&fl=*&wt=json')
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        return response.json()

    def get(self, hal_id: int):
        """
        :param hal_id: the HAL ID of adocument
        :return: response from HAL API
        """
        full_url = urljoin(self.url, f'hal?q=docid:{hal_id}&fl=*&wt=json')
        response = self.session.get(full_url)
        self.last_url_requested = full_url
        response.raise_for_status()
        return response.json()




import os
from unittest import TestCase

from dabeplech.parsers.kegg.list import KeggPathwayListParser
from dabeplech.parsers.kegg.pathway import KeggPathwayParser


class TestKeggPathwayParser(TestCase):

    def _get_tested_entry(self, file_path):
        input_path = os.path.join(os.path.dirname(__file__), file_path)
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggPathwayParser(test_txt)
        test_parser.parse()
        return test_parser.validated_entry

    def _common_tests(self, tested_entry):
        # Test some of the attribute and content of the file
        self.assertIn('Tyrosine metabolism', tested_entry.names)
        # Test some keys of dictionnary
        self.assertIn('M00042', tested_entry.modules.keys())
        self.assertIn('GO', tested_entry.dblinks.dict().keys())
        self.assertIn('H00163', tested_entry.diseases.keys())
        # Test references
        self.assertEqual(len(tested_entry.references), 4)
        self.assertEqual(tested_entry.references[0].pubmed_id, 8550403)
        self.assertEqual(tested_entry.references[0].doi, "10.1128/JB.178.1.111-120.1996")

    def test_parsing_ko(self):
        tested_entry = self._get_tested_entry('files/ko00350.txt')
        self._common_tests(tested_entry)
        # Test specifics
        self.assertEqual(tested_entry.entry_id, 'ko00350')
        # Keys of dict
        self.assertIn('ko00020', tested_entry.related_pathways.keys())
        self.assertIn('C00022', tested_entry.compounds.keys())
        self.assertIn('K14454', tested_entry.orthologies.keys())
        self.assertIn('ko00350', tested_entry.pathway_maps.keys())

    def test_parsing_map(self):
        tested_entry = self._get_tested_entry('files/map00350.txt')
        self._common_tests(tested_entry)
        # Test specifics
        self.assertEqual(tested_entry.entry_id, 'map00350')
        self.assertEqual(tested_entry.ko_pathway, 'ko00350')
        # List
        self.assertIn('Amino acid metabolism', tested_entry.classes)
        # Keys of dict
        self.assertIn('map00020', tested_entry.related_pathways.keys())
        self.assertIn('map00350', tested_entry.pathway_maps.keys())

    def test_parsing_map_with_ref_no_pubmedid(self):
        tested_entry = self._get_tested_entry('files/map00030.txt')
        # Test some of the attribute and content of the file
        self.assertIn('Pentose phosphate pathway', tested_entry.names)
        # Test some keys of dictionnary
        self.assertIn('M00004', tested_entry.modules.keys())
        self.assertIn('GO', tested_entry.dblinks.dict().keys())
        self.assertIn('H00196', tested_entry.diseases.keys())
        # Test references
        self.assertEqual(len(tested_entry.references), 9)
        self.assertEqual(tested_entry.references[0].title, "[Metabolic Maps] (In Japanese)")
        # Test specifics
        self.assertEqual(tested_entry.entry_id, 'map00030')
        self.assertEqual(tested_entry.ko_pathway, 'ko00030')
        # List
        self.assertIn('Metabolism', tested_entry.classes)
        # Keys of dict
        self.assertIn('map00052', tested_entry.related_pathways.keys())
        self.assertIn('map00030', tested_entry.pathway_maps.keys())


class TestKeggPathwayListParser(TestCase):

    def test_parsing(self):
        input_path = os.path.join(os.path.dirname(__file__), 'files/list_pathway.txt')
        test_file = open(input_path, 'r')
        test_txt = test_file.read()
        test_parser = KeggPathwayListParser(test_txt)
        test_parser.parse()
        tested_model = test_parser.validated_model
        self.assertEqual(len(tested_model.entries), 10)
        # Testing third entry of the list
        tested_entry = tested_model.entries[2]
        self.assertEqual(tested_entry.entry_id, 'map00030')
        self.assertIn('Pentose phosphate pathway', tested_entry.names)
        # Test entry has only expected
        for k in vars(tested_entry).keys():
            self.assertIn(k, ['names', 'entry_id'])



from typing import List

from pydantic import BaseModel, Field

from dabeplech.models.kegg.dblinks import DBLinksModel
from .base import BaseKeggModel, BaseKeggWithRefModel


class KeggOrthologyModel(BaseKeggWithRefModel):
    entry_id: str = Field(regex=r"^K\d{5}$")
    definition: str = None
    modules: dict = None
    ec_numbers: List[str] = None  # noqa
    pathways: dict = None
    genes: dict = None
    dblinks: DBLinksModel = None
    diseases: dict = None


class LightKeggOrthologyModel(BaseKeggModel):
    definition: str = None
    ec_numbers: List[str] = None  # noqa


class KeggOrthologyListModel(BaseModel):
    entries: List[LightKeggOrthologyModel]



import logging

from dabeplech.models.kegg.module import KeggModuleModel
from dabeplech.parsers.kegg.base import BaseKeggParser

logging.basicConfig()
logger = logging.getLogger()


class KeggModuleParser(BaseKeggParser):
    """
    Parser for KEGG module `plain text` result from KEGG API.
    """
    model = KeggModuleModel



from abc import ABC, abstractmethod

from pydantic import BaseModel


class BaseParser(ABC):
    """
    Base structure for parsers
    """
    model = BaseModel  # Pydantic model to describe the response

    def __init__(self, content_response: str):
        """
        :param content_response: content response from the API
        """
        self.parsed_content = {}
        self.lines = content_response.rstrip().split('\n')

    @abstractmethod
    def parse(self):
        """
        Perform parsing of the ``content_response``
        """
        pass

    @property
    def validated_entry(self):
        """
        This property aims to return the entry, validated with the model

        :return: Validated entry.
        """
        return self.model(**self.parsed_content)


class BaseListParser(ABC):
    """
    Base structure for parsers
    """
    model = BaseModel  # Pydantic model to describe the response

    def __init__(self, content_response: str):
        """
        :param content_response: content response from the API
        """
        self.parsed_content = []
        self.lines = content_response.rstrip().split('\n')

    @abstractmethod
    def parse(self):
        """
        Perform parsing of the ``content_response``
        """
        pass

    @property
    def validated_model(self):
        """
        This property aims to return the model list, with the list accessible through entries attribute
        """
        return self.model(entries=self.parsed_content)



from setuptools import setup, find_packages

exec(open('dabeplech/version.py').read())

setup(
    name="dabeplech",
    version=__version__,  # noqa
    description='Light library to perform request to different bioinformatics APIs',
    author='Kenzo-Hugo Hillion',
    author_email='kehillio@pasteur.fr',
    install_requires=[
        'pydantic==1.5.1',
        'requests==2.23.0',
        'colored==1.4.2',
        'beautifulsoup4==4.9.1'
    ],
    packages=find_packages()
)



from urllib.parse import urljoin

import requests


class BaseAPI:
    BASE_URL = ''
    ROUTE = ''
    HEADERS = {
        'Content-type': 'application/json',
        'Accept': '*/*'
    }
    SESSION = requests.Session

    def __init__(self):
        if not getattr(self, 'base_url', None):
            self.base_url = self.BASE_URL
        if not getattr(self, 'route', None):
            self.route = self.ROUTE
        self.url = urljoin(self.base_url, self.route)
        self.last_url_requested = None
        self.session = self.SESSION()
        self.session.headers.update(self.HEADERS)


class LISTMixin:
    """
    Corresponds to a ``GET`` that retrieve all items
    """

    def list(self, params: dict = None):
        """
        Perform GET request to the service

        :param params: query params for the request
        """
        response = self.session.get(self.url, params=params)
        self.last_url_requested = self.url
        response.raise_for_status()
        return response.json()


class GETMixin:
    """
    Corresponds to a ``GET`` that retrieve one item from its ID
    """

    def get(self, entry_id: str, params: dict = None):
        """
        Perform GET request to the service

        :param entry_id: ID of the entry you want to retrieve
        :param params: query params for the request
        """
        full_url = urljoin(self.url, entry_id)
        response = self.session.get(full_url, params=params)
        self.last_url_requested = full_url
        response.raise_for_status()
        return response.json()


class POSTMixin:

    def post(self, data: dict):
        """
        Perform POST request to the service

        :param data: data to send in the body of your POST
        """
        response = self.session.post(f"{self.url}", json=data)
        self.last_url_requested = self.url
        response.raise_for_status()
        return response.json()


class PUTMixin:

    def put(self, data: dict, entry_id: str = None):
        """
        Perform PUT request to the service

        :param data: data to send in the body of your PUT
        :param entry_id: ID of the entry you want to update
        """
        if entry_id:
            full_url = urljoin(self.url, entry_id + '/')
        else:
            full_url = self.url
        response = self.session.put(f"{full_url}", json=data)
        self.last_url_requested = full_url
        response.raise_for_status()
        return response.json()



import logging

logging.basicConfig()
logger = logging.getLogger()


class KeggReferenceParser:
    """
    Simple class that handle parsing of the REFERENCE part from KEGG API

    It is build with the idea of setter handling each line directly

    e.g.:
    ```
    parser.pubmed_id = 'REFERENCE    PMID:124142"
    print(parser.pubmed_id)
    124142
    ```
    """

    @property
    def pubmed_id(self):
        return self._pubmed_id

    @pubmed_id.setter
    def pubmed_id(self, line: str):
        """
        :param line: corresponding line
        :type line: STR
        """
        if 'PMID' not in line:
            logger.warning("no PMID for the reference. Corresponding line: %s", line)
            self._pubmed_id = None
        else:
            self._pubmed_id = line.split()[1].split('PMID:')[-1].strip()

    @property
    def authors(self):
        return self._authors

    @authors.setter
    def authors(self, line):
        all_authors = line.split(maxsplit=1)[-1].split(',')
        self._authors = [author.strip() for author in all_authors]

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, line):
        self._title = line.split(maxsplit=1)[-1]

    @property
    def journal(self):
        return self._journal

    @journal.setter
    def journal(self, line):
        self._journal = line.split(maxsplit=1)[-1]

    @property
    def doi(self):
        return self._doi

    @doi.setter
    def doi(self, line):
        self._doi = line.split('DOI:', maxsplit=1)[-1]

    def dict(self):
        return {
            'pubmed_id': getattr(self, 'pubmed_id', None),
            'authors': getattr(self, 'authors', None),
            'title': getattr(self, 'title', None),
            'journal': getattr(self, 'journal', None),
            'doi': getattr(self, 'doi', None),
        }



from .link import KeggLinkParser  # noqa
from .list import (  # noqa
    KeggOrthologyListParser, KeggPathwayListParser, KeggModuleListParser
)
from .module import KeggModuleParser  # noqa
from .orthology import KeggOrthologyParser  # noqa
from .pathway import KeggPathwayParser  # noqa
